# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the number of spaces after hyphens (``-``).

.. rubric:: Options

* ``max-spaces-after`` defines the maximal number of spaces allowed after
  hyphens.

.. rubric:: Default values (when enabled)

.. code-block:: yaml

 rules:
   hyphens:
     max-spaces-after: 1

.. rubric:: Examples

#. With ``hyphens: {max-spaces-after: 1}``

   the following code snippet would **PASS**:
   ::

    - first list:
        - a
        - b
    - - 1
      - 2
      - 3

   the following code snippet would **FAIL**:
   ::

    -  first list:
         - a
         - b

   the following code snippet would **FAIL**:
   ::

    - - 1
      -  2
      - 3

#. With ``hyphens: {max-spaces-after: 3}``

   the following code snippet would **PASS**:
   ::

    -   key
    -  key2
    - key42

   the following code snippet would **FAIL**:
   ::

    -    key
    -   key2
    -  key42
"""


import yaml

from yamllint.rules.common import spaces_after


ID = 'hyphens'
TYPE = 'token'
CONF = {'max-spaces-after': int}
DEFAULT = {'max-spaces-after': 1}


def check(conf, token, prev, next, nextnext, context):
    if isinstance(token, yaml.BlockEntryToken):
        problem = spaces_after(token, prev, next,
                               max=conf['max-spaces-after'],
                               max_desc='too many spaces after hyphen')
        if problem is not None:
            yield problem



# Copyright (C) 2017 Johannes F. Knauf
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to enforce alphabetical ordering of keys in mappings. The sorting
order uses the Unicode code point number as a default. As a result, the
ordering is case-sensitive and not accent-friendly (see examples below).
This can be changed by setting the global ``locale`` option.  This allows one
to sort case and accents properly.

.. rubric:: Examples

#. With ``key-ordering: {}``

   the following code snippet would **PASS**:
   ::

    - key 1: v
      key 2: val
      key 3: value
    - {a: 1, b: 2, c: 3}
    - T-shirt: 1
      T-shirts: 2
      t-shirt: 3
      t-shirts: 4
    - hair: true
      hais: true
      haïr: true
      haïssable: true

   the following code snippet would **FAIL**:
   ::

    - key 2: v
      key 1: val

   the following code snippet would **FAIL**:
   ::

    - {b: 1, a: 2}

   the following code snippet would **FAIL**:
   ::

    - T-shirt: 1
      t-shirt: 2
      T-shirts: 3
      t-shirts: 4

   the following code snippet would **FAIL**:
   ::

    - haïr: true
      hais: true

#. With global option ``locale: "en_US.UTF-8"`` and rule ``key-ordering: {}``

   as opposed to before, the following code snippet would now **PASS**:
   ::

    - t-shirt: 1
      T-shirt: 2
      t-shirts: 3
      T-shirts: 4
    - hair: true
      haïr: true
      hais: true
      haïssable: true
"""

from locale import strcoll

import yaml

from yamllint.linter import LintProblem


ID = 'key-ordering'
TYPE = 'token'

MAP, SEQ = range(2)


class Parent:
    def __init__(self, type):
        self.type = type
        self.keys = []


def check(conf, token, prev, next, nextnext, context):
    if 'stack' not in context:
        context['stack'] = []

    if isinstance(token, (yaml.BlockMappingStartToken,
                          yaml.FlowMappingStartToken)):
        context['stack'].append(Parent(MAP))
    elif isinstance(token, (yaml.BlockSequenceStartToken,
                            yaml.FlowSequenceStartToken)):
        context['stack'].append(Parent(SEQ))
    elif isinstance(token, (yaml.BlockEndToken,
                            yaml.FlowMappingEndToken,
                            yaml.FlowSequenceEndToken)):
        context['stack'].pop()
    elif (isinstance(token, yaml.KeyToken) and
          isinstance(next, yaml.ScalarToken)):
        # This check is done because KeyTokens can be found inside flow
        # sequences... strange, but allowed.
        if len(context['stack']) > 0 and context['stack'][-1].type == MAP:
            if any(strcoll(next.value, key) < 0
                   for key in context['stack'][-1].keys):
                yield LintProblem(
                    next.start_mark.line + 1, next.start_mark.column + 1,
                    'wrong ordering of key "%s" in mapping' % next.value)
            else:
                context['stack'][-1].keys.append(next.value)



# Copyright (C) 2023 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class AnchorsTestCase(RuleTestCase):
    rule_id = 'anchors'

    def test_disabled(self):
        conf = 'anchors: disable'
        self.check('---\n'
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- &f_m {k: v}\n'
                   '- &f_s [1, 2]\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '- *f_m\n'
                   '- *f_s\n'
                   '---\n'  # redeclare anchors in a new document
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &y 3, *x: 4, e: *y}\n'
                   '...\n', conf)
        self.check('---\n'
                   '- &i 42\n'
                   '---\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'    # declared in a previous document
                   '- *f_m\n'  # never declared
                   '- *f_m\n'
                   '- *f_m\n'
                   '- *f_s\n'  # declared after
                   '- &f_s [1, 2]\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   '---\n'
                   'block mapping 1: &b_m_bis\n'
                   '  key: value\n'
                   'block mapping 2: &b_m_bis\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &x 3, *x: 4, e: *y}\n'
                   '...\n', conf)

    def test_forbid_undeclared_aliases(self):
        conf = ('anchors:\n'
                '  forbid-undeclared-aliases: true\n'
                '  forbid-duplicated-anchors: false\n')
        self.check('---\n'
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- &f_m {k: v}\n'
                   '- &f_s [1, 2]\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '- *f_m\n'
                   '- *f_s\n'
                   '---\n'  # redeclare anchors in a new document
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &y 3, *x: 4, e: *y}\n'
                   '...\n', conf)
        self.check('---\n'
                   '- &i 42\n'
                   '---\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'    # declared in a previous document
                   '- *f_m\n'  # never declared
                   '- *f_m\n'
                   '- *f_m\n'
                   '- *f_s\n'  # declared after
                   '- &f_s [1, 2]\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   '---\n'
                   'block mapping 1: &b_m_bis\n'
                   '  key: value\n'
                   'block mapping 2: &b_m_bis\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &x 3, *x: 4, e: *y}\n'
                   '...\n', conf,
                   problem1=(9, 3),
                   problem2=(10, 3),
                   problem3=(11, 3),
                   problem4=(12, 3),
                   problem5=(13, 3),
                   problem6=(24, 7),
                   problem7=(27, 36))

    def test_forbid_duplicated_anchors(self):
        conf = ('anchors:\n'
                '  forbid-undeclared-aliases: false\n'
                '  forbid-duplicated-anchors: true\n')
        self.check('---\n'
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- &f_m {k: v}\n'
                   '- &f_s [1, 2]\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '- *f_m\n'
                   '- *f_s\n'
                   '---\n'  # redeclare anchors in a new document
                   '- &b true\n'
                   '- &i 42\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'
                   '- *s\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &y 3, *x: 4, e: *y}\n'
                   '...\n', conf)
        self.check('---\n'
                   '- &i 42\n'
                   '---\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &b true\n'
                   '- &s hello\n'
                   '- *b\n'
                   '- *i\n'    # declared in a previous document
                   '- *f_m\n'  # never declared
                   '- *f_m\n'
                   '- *f_m\n'
                   '- *f_s\n'  # declared after
                   '- &f_s [1, 2]\n'
                   '---\n'
                   'block mapping: &b_m\n'
                   '  key: value\n'
                   '---\n'
                   'block mapping 1: &b_m_bis\n'
                   '  key: value\n'
                   'block mapping 2: &b_m_bis\n'
                   '  key: value\n'
                   'extended:\n'
                   '  <<: *b_m\n'
                   '  foo: bar\n'
                   '---\n'
                   '{a: 1, &x b: 2, c: &x 3, *x: 4, e: *y}\n'
                   '...\n', conf,
                   problem1=(5, 3),
                   problem2=(6, 3),
                   problem3=(21, 18),
                   problem4=(27, 20))



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class KeyDuplicatesTestCase(RuleTestCase):
    rule_id = 'key-duplicates'

    def test_disabled(self):
        conf = 'key-duplicates: disable'
        self.check('---\n'
                   'block mapping:\n'
                   '  key: a\n'
                   '  otherkey: b\n'
                   '  key: c\n', conf)
        self.check('---\n'
                   'flow mapping:\n'
                   '  {key: a, otherkey: b, key: c}\n', conf)
        self.check('---\n'
                   'duplicated twice:\n'
                   '  - k: a\n'
                   '    ok: b\n'
                   '    k: c\n'
                   '    k: d\n', conf)
        self.check('---\n'
                   'duplicated twice:\n'
                   '  - {k: a, ok: b, k: c, k: d}\n', conf)
        self.check('---\n'
                   'multiple duplicates:\n'
                   '  a: 1\n'
                   '  b: 2\n'
                   '  c: 3\n'
                   '  d: 4\n'
                   '  d: 5\n'
                   '  b: 6\n', conf)
        self.check('---\n'
                   'multiple duplicates:\n'
                   '  {a: 1, b: 2, c: 3, d: 4, d: 5, b: 6}\n', conf)
        self.check('---\n'
                   'at: root\n'
                   'multiple: times\n'
                   'at: root\n', conf)
        self.check('---\n'
                   'nested but OK:\n'
                   '  a: {a: {a: 1}}\n'
                   '  b:\n'
                   '    b: 2\n'
                   '    c: 3\n', conf)
        self.check('---\n'
                   'nested duplicates:\n'
                   '  a: {a: 1, a: 1}\n'
                   '  b:\n'
                   '    c: 3\n'
                   '    d: 4\n'
                   '    d: 4\n'
                   '  b: 2\n', conf)
        self.check('---\n'
                   'duplicates with many styles: 1\n'
                   '"duplicates with many styles": 1\n'
                   '\'duplicates with many styles\': 1\n'
                   '? duplicates with many styles\n'
                   ': 1\n'
                   '? >-\n'
                   '    duplicates with\n'
                   '    many styles\n'
                   ': 1\n', conf)
        self.check('---\n'
                   'Merge Keys are OK:\n'
                   'anchor_one: &anchor_one\n'
                   '  one: one\n'
                   'anchor_two: &anchor_two\n'
                   '  two: two\n'
                   'anchor_reference:\n'
                   '  <<: *anchor_one\n'
                   '  <<: *anchor_two\n', conf)
        self.check('---\n'
                   '{a: 1, b: 2}}\n', conf, problem=(2, 13, 'syntax'))
        self.check('---\n'
                   '[a, b, c]]\n', conf, problem=(2, 10, 'syntax'))

    def test_enabled(self):
        conf = 'key-duplicates: enable'
        self.check('---\n'
                   'block mapping:\n'
                   '  key: a\n'
                   '  otherkey: b\n'
                   '  key: c\n', conf,
                   problem=(5, 3))
        self.check('---\n'
                   'flow mapping:\n'
                   '  {key: a, otherkey: b, key: c}\n', conf,
                   problem=(3, 25))
        self.check('---\n'
                   'duplicated twice:\n'
                   '  - k: a\n'
                   '    ok: b\n'
                   '    k: c\n'
                   '    k: d\n', conf,
                   problem1=(5, 5), problem2=(6, 5))
        self.check('---\n'
                   'duplicated twice:\n'
                   '  - {k: a, ok: b, k: c, k: d}\n', conf,
                   problem1=(3, 19), problem2=(3, 25))
        self.check('---\n'
                   'multiple duplicates:\n'
                   '  a: 1\n'
                   '  b: 2\n'
                   '  c: 3\n'
                   '  d: 4\n'
                   '  d: 5\n'
                   '  b: 6\n', conf,
                   problem1=(7, 3), problem2=(8, 3))
        self.check('---\n'
                   'multiple duplicates:\n'
                   '  {a: 1, b: 2, c: 3, d: 4, d: 5, b: 6}\n', conf,
                   problem1=(3, 28), problem2=(3, 34))
        self.check('---\n'
                   'at: root\n'
                   'multiple: times\n'
                   'at: root\n', conf,
                   problem=(4, 1))
        self.check('---\n'
                   'nested but OK:\n'
                   '  a: {a: {a: 1}}\n'
                   '  b:\n'
                   '    b: 2\n'
                   '    c: 3\n', conf)
        self.check('---\n'
                   'nested duplicates:\n'
                   '  a: {a: 1, a: 1}\n'
                   '  b:\n'
                   '    c: 3\n'
                   '    d: 4\n'
                   '    d: 4\n'
                   '  b: 2\n', conf,
                   problem1=(3, 13), problem2=(7, 5), problem3=(8, 3))
        self.check('---\n'
                   'duplicates with many styles: 1\n'
                   '"duplicates with many styles": 1\n'
                   '\'duplicates with many styles\': 1\n'
                   '? duplicates with many styles\n'
                   ': 1\n'
                   '? >-\n'
                   '    duplicates with\n'
                   '    many styles\n'
                   ': 1\n', conf,
                   problem1=(3, 1), problem2=(4, 1), problem3=(5, 3),
                   problem4=(7, 3))
        self.check('---\n'
                   'Merge Keys are OK:\n'
                   'anchor_one: &anchor_one\n'
                   '  one: one\n'
                   'anchor_two: &anchor_two\n'
                   '  two: two\n'
                   'anchor_reference:\n'
                   '  <<: *anchor_one\n'
                   '  <<: *anchor_two\n', conf)
        self.check('---\n'
                   '{a: 1, b: 2}}\n', conf, problem=(2, 13, 'syntax'))
        self.check('---\n'
                   '[a, b, c]]\n', conf, problem=(2, 10, 'syntax'))

    def test_key_tokens_in_flow_sequences(self):
        conf = 'key-duplicates: enable'
        self.check('---\n'
                   '[\n'
                   '  flow: sequence, with, key: value, mappings\n'
                   ']\n', conf)



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class YamllintDirectivesTestCase(RuleTestCase):
    conf = ('commas: disable\n'
            'trailing-spaces: {}\n'
            'colons: {max-spaces-before: 1}\n')

    def test_disable_directive(self):
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'),
                   problem3=(6, 7, 'colons'),
                   problem4=(6, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '# yamllint disable\n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem=(3, 18, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint enable\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(8, 7, 'colons'),
                   problem2=(8, 26, 'trailing-spaces'))

    def test_disable_directive_with_rules(self):
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '# yamllint disable rule:trailing-spaces\n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(5, 8, 'colons'),
                   problem3=(7, 7, 'colons'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable rule:trailing-spaces\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint enable rule:trailing-spaces\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(5, 8, 'colons'),
                   problem2=(8, 7, 'colons'),
                   problem3=(8, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable rule:trailing-spaces\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint enable\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(5, 8, 'colons'),
                   problem2=(8, 7, 'colons'),
                   problem3=(8, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint enable rule:trailing-spaces\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem=(8, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable rule:colons\n'
                   '- trailing spaces    \n'
                   '# yamllint disable rule:trailing-spaces\n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint enable rule:colons\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(4, 18, 'trailing-spaces'),
                   problem2=(9, 7, 'colons'))

    def test_disable_line_directive(self):
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '# yamllint disable-line\n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(7, 7, 'colons'),
                   problem3=(7, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon  # yamllint disable-line\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(6, 7, 'colons'),
                   problem3=(6, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]  # yamllint disable-line\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'),
                   problem3=(6, 7, 'colons'),
                   problem4=(6, 26, 'trailing-spaces'))

    def test_disable_line_directive_with_rules(self):
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable-line rule:colons\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(4, 18, 'trailing-spaces'),
                   problem2=(5, 8, 'colons'),
                   problem3=(7, 7, 'colons'),
                   problem4=(7, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces  # yamllint disable-line rule:colons  \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 55, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'),
                   problem3=(6, 7, 'colons'),
                   problem4=(6, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '# yamllint disable-line rule:colons\n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(7, 7, 'colons'),
                   problem3=(7, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon  # yamllint disable-line rule:colons\n'
                   '- [valid , YAML]\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(6, 7, 'colons'),
                   problem3=(6, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable-line rule:colons\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'),
                   problem3=(7, 26, 'trailing-spaces'))
        self.check('---\n'
                   '- [valid , YAML]\n'
                   '- trailing spaces    \n'
                   '- bad   : colon\n'
                   '- [valid , YAML]\n'
                   '# yamllint disable-line rule:colons rule:trailing-spaces\n'
                   '- bad  : colon and spaces   \n'
                   '- [valid , YAML]\n',
                   self.conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'))

    def test_disable_directive_with_rules_and_dos_lines(self):
        conf = self.conf + 'new-lines: {type: dos}\n'
        self.check('---\r\n'
                   '- [valid , YAML]\r\n'
                   '# yamllint disable rule:trailing-spaces\r\n'
                   '- trailing spaces    \r\n'
                   '- bad   : colon\r\n'
                   '- [valid , YAML]\r\n'
                   '# yamllint enable rule:trailing-spaces\r\n'
                   '- bad  : colon and spaces   \r\n'
                   '- [valid , YAML]\r\n',
                   conf,
                   problem1=(5, 8, 'colons'),
                   problem2=(8, 7, 'colons'),
                   problem3=(8, 26, 'trailing-spaces'))
        self.check('---\r\n'
                   '- [valid , YAML]\r\n'
                   '- trailing spaces    \r\n'
                   '- bad   : colon\r\n'
                   '- [valid , YAML]\r\n'
                   '# yamllint disable-line rule:colons\r\n'
                   '- bad  : colon and spaces   \r\n'
                   '- [valid , YAML]\r\n',
                   conf,
                   problem1=(3, 18, 'trailing-spaces'),
                   problem2=(4, 8, 'colons'),
                   problem3=(7, 26, 'trailing-spaces'))

    def test_directive_on_last_line(self):
        conf = 'new-line-at-end-of-file: {}'
        self.check('---\n'
                   'no new line',
                   conf,
                   problem=(2, 12, 'new-line-at-end-of-file'))
        self.check('---\n'
                   '# yamllint disable\n'
                   'no new line',
                   conf)
        self.check('---\n'
                   'no new line  # yamllint disable',
                   conf)

    def test_indented_directive(self):
        conf = 'brackets: {min-spaces-inside: 0, max-spaces-inside: 0}'
        self.check('---\n'
                   '- a: 1\n'
                   '  b:\n'
                   '    c: [    x]\n',
                   conf,
                   problem=(4, 12, 'brackets'))
        self.check('---\n'
                   '- a: 1\n'
                   '  b:\n'
                   '    # yamllint disable-line rule:brackets\n'
                   '    c: [    x]\n',
                   conf)

    def test_directive_on_itself(self):
        conf = ('comments: {min-spaces-from-content: 2}\n'
                'comments-indentation: {}\n')
        self.check('---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf,
                   problem1=(2, 8, 'comments'),
                   problem2=(4, 2, 'comments-indentation'))
        self.check('---\n'
                   '# yamllint disable\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('---\n'
                   '- a: 1 # yamllint disable-line\n'
                   '  b:\n'
                   '    # yamllint disable-line\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('---\n'
                   '- a: 1 # yamllint disable-line rule:comments\n'
                   '  b:\n'
                   '    # yamllint disable-line rule:comments-indentation\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('---\n'
                   '# yamllint disable\n'
                   '- a: 1 # comment too close\n'
                   '  # yamllint enable rule:comments-indentation\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf,
                   problem=(6, 2, 'comments-indentation'))

    def test_disable_file_directive(self):
        conf = ('comments: {min-spaces-from-content: 2}\n'
                'comments-indentation: {}\n')
        self.check('# yamllint disable-file\n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('#    yamllint disable-file\n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('#yamllint disable-file\n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('#yamllint disable-file    \n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf)
        self.check('---\n'
                   '# yamllint disable-file\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf,
                   problem1=(3, 8, 'comments'),
                   problem2=(5, 2, 'comments-indentation'))
        self.check('# yamllint disable-file: rules cannot be specified\n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf,
                   problem1=(3, 8, 'comments'),
                   problem2=(5, 2, 'comments-indentation'))
        self.check('AAAA yamllint disable-file\n'
                   '---\n'
                   '- a: 1 # comment too close\n'
                   '  b:\n'
                   ' # wrong indentation\n'
                   '    c: [x]\n',
                   conf,
                   problem1=(1, 1, 'document-start'),
                   problem2=(3, 8, 'comments'),
                   problem3=(5, 2, 'comments-indentation'))

    def test_disable_file_directive_not_at_first_position(self):
        self.check('# yamllint disable-file\n'
                   '---\n'
                   '- bad  : colon and spaces   \n',
                   self.conf)
        self.check('---\n'
                   '# yamllint disable-file\n'
                   '- bad  : colon and spaces   \n',
                   self.conf,
                   problem1=(3, 7, 'colons'),
                   problem2=(3, 26, 'trailing-spaces'))

    def test_disable_file_directive_with_syntax_error(self):
        self.check('# This file is not valid YAML (it is a Jinja template)\n'
                   '{% if extra_info %}\n'
                   'key1: value1\n'
                   '{% endif %}\n'
                   'key2: value2\n',
                   self.conf,
                   problem=(2, 2, 'syntax'))
        self.check('# yamllint disable-file\n'
                   '# This file is not valid YAML (it is a Jinja template)\n'
                   '{% if extra_info %}\n'
                   'key1: value1\n'
                   '{% endif %}\n'
                   'key2: value2\n',
                   self.conf)

    def test_disable_file_directive_with_dos_lines(self):
        self.check('# yamllint disable-file\r\n'
                   '---\r\n'
                   '- bad  : colon and spaces   \r\n',
                   self.conf)
        self.check('# yamllint disable-file\r\n'
                   '# This file is not valid YAML (it is a Jinja template)\r\n'
                   '{% if extra_info %}\r\n'
                   'key1: value1\r\n'
                   '{% endif %}\r\n'
                   'key2: value2\r\n',
                   self.conf)



# Copyright (C) 2017 Greg Dubicki
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to prevent nodes with empty content, that implicitly result in
``null`` values.

.. rubric:: Options

* Use ``forbid-in-block-mappings`` to prevent empty values in block mappings.
* Use ``forbid-in-flow-mappings`` to prevent empty values in flow mappings.

.. rubric:: Default values (when enabled)

.. code-block:: yaml

 rules:
   empty-values:
     forbid-in-block-mappings: true
     forbid-in-flow-mappings: true

.. rubric:: Examples

#. With ``empty-values: {forbid-in-block-mappings: true}``

   the following code snippets would **PASS**:
   ::

    some-mapping:
      sub-element: correctly indented

   ::

    explicitly-null: null

   the following code snippets would **FAIL**:
   ::

    some-mapping:
    sub-element: incorrectly indented

   ::

    implicitly-null:

#. With ``empty-values: {forbid-in-flow-mappings: true}``

   the following code snippet would **PASS**:
   ::

    {prop: null}
    {a: 1, b: 2, c: 3}

   the following code snippets would **FAIL**:
   ::

    {prop: }

   ::

    {a: 1, b:, c: 3}

"""

import yaml

from yamllint.linter import LintProblem


ID = 'empty-values'
TYPE = 'token'
CONF = {'forbid-in-block-mappings': bool,
        'forbid-in-flow-mappings': bool}
DEFAULT = {'forbid-in-block-mappings': True,
           'forbid-in-flow-mappings': True}


def check(conf, token, prev, next, nextnext, context):

    if conf['forbid-in-block-mappings']:
        if isinstance(token, yaml.ValueToken) and isinstance(next, (
                yaml.KeyToken, yaml.BlockEndToken)):
            yield LintProblem(token.start_mark.line + 1,
                              token.end_mark.column + 1,
                              'empty value in block mapping')

    if conf['forbid-in-flow-mappings']:
        if isinstance(token, yaml.ValueToken) and isinstance(next, (
                yaml.FlowEntryToken, yaml.FlowMappingEndToken)):
            yield LintProblem(token.start_mark.line + 1,
                              token.end_mark.column + 1,
                              'empty value in flow mapping')



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class ColonTestCase(RuleTestCase):
    rule_id = 'colons'

    def test_disabled(self):
        conf = 'colons: disable'
        self.check('---\n'
                   'object:\n'
                   '  k1 : v1\n'
                   'obj2:\n'
                   '  k2     :\n'
                   '    - 8\n'
                   '  k3:\n'
                   '    val\n'
                   '  property   : value\n'
                   '  prop2      : val2\n'
                   '  propriété  : [valeur]\n'
                   '  o:\n'
                   '    k1: [v1, v2]\n'
                   '  p:\n'
                   '    - k3: >\n'
                   '        val\n'
                   '    - o: {k1: v1}\n'
                   '    - p: kdjf\n'
                   '    - q: val0\n'
                   '    - q2:\n'
                   '        - val1\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k1:   v1\n'
                   'obj2:\n'
                   '  k2:\n'
                   '    - 8\n'
                   '  k3:\n'
                   '    val\n'
                   '  property:     value\n'
                   '  prop2:        val2\n'
                   '  propriété:    [valeur]\n'
                   '  o:\n'
                   '    k1:  [v1, v2]\n', conf)
        self.check('---\n'
                   'obj:\n'
                   '  p:\n'
                   '    - k1: >\n'
                   '        val\n'
                   '    - k3:  >\n'
                   '        val\n'
                   '    - o: {k1: v1}\n'
                   '    - o:  {k1: v1}\n'
                   '    - q2:\n'
                   '        - val1\n'
                   '...\n', conf)
        self.check('---\n'
                   'a: {b: {c:  d, e : f}}\n', conf)

    def test_before_enabled(self):
        conf = 'colons: {max-spaces-before: 0, max-spaces-after: -1}'
        self.check('---\n'
                   'object:\n'
                   '  k1:\n'
                   '    - a\n'
                   '    - b\n'
                   '  k2: v2\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k1 :\n'
                   '    - a\n'
                   '    - b\n'
                   '  k2: v2\n'
                   '...\n', conf, problem=(3, 5))
        self.check('---\n'
                   'lib :\n'
                   '  - var\n'
                   '...\n', conf, problem=(2, 4))
        self.check('---\n'
                   '- lib :\n'
                   '    - var\n'
                   '...\n', conf, problem=(2, 6))
        self.check('---\n'
                   'a: {b: {c : d, e : f}}\n', conf,
                   problem1=(2, 10), problem2=(2, 17))

    def test_before_max(self):
        conf = 'colons: {max-spaces-before: 3, max-spaces-after: -1}'
        self.check('---\n'
                   'object :\n'
                   '  k1   :\n'
                   '    - a\n'
                   '    - b\n'
                   '  k2  : v2\n'
                   '...\n', conf)
        self.check('---\n'
                   'object :\n'
                   '  k1    :\n'
                   '    - a\n'
                   '    - b\n'
                   '  k2  : v2\n'
                   '...\n', conf, problem=(3, 8))

    def test_before_with_explicit_block_mappings(self):
        conf = 'colons: {max-spaces-before: 0, max-spaces-after: 1}'
        self.check('---\n'
                   'object:\n'
                   '  ? key\n'
                   '  : value\n'
                   '...\n', conf)
        self.check('---\n'
                   'object :\n'
                   '  ? key\n'
                   '  : value\n'
                   '...\n', conf, problem=(2, 7))
        self.check('---\n'
                   '? >\n'
                   '    multi-line\n'
                   '    key\n'
                   ': >\n'
                   '    multi-line\n'
                   '    value\n'
                   '...\n', conf)
        self.check('---\n'
                   '- ? >\n'
                   '      multi-line\n'
                   '      key\n'
                   '  : >\n'
                   '      multi-line\n'
                   '      value\n'
                   '...\n', conf)
        self.check('---\n'
                   '- ? >\n'
                   '      multi-line\n'
                   '      key\n'
                   '  :  >\n'
                   '       multi-line\n'
                   '       value\n'
                   '...\n', conf, problem=(5, 5))

    def test_after_enabled(self):
        conf = 'colons: {max-spaces-before: -1, max-spaces-after: 1}'
        self.check('---\n'
                   'key: value\n', conf)
        self.check('---\n'
                   'key:  value\n', conf, problem=(2, 6))
        self.check('---\n'
                   'object:\n'
                   '  k1:  [a, b]\n'
                   '  k2: string\n', conf, problem=(3, 7))
        self.check('---\n'
                   'object:\n'
                   '  k1: [a, b]\n'
                   '  k2:  string\n', conf, problem=(4, 7))
        self.check('---\n'
                   'object:\n'
                   '  other: {key:  value}\n'
                   '...\n', conf, problem=(3, 16))
        self.check('---\n'
                   'a: {b: {c:  d, e :  f}}\n', conf,
                   problem1=(2, 12), problem2=(2, 20))

    def test_after_enabled_question_mark(self):
        conf = 'colons: {max-spaces-before: -1, max-spaces-after: 1}'
        self.check('---\n'
                   '? key\n'
                   ': value\n', conf)
        self.check('---\n'
                   '?  key\n'
                   ': value\n', conf, problem=(2, 3))
        self.check('---\n'
                   '?  key\n'
                   ':  value\n', conf, problem1=(2, 3), problem2=(3, 3))
        self.check('---\n'
                   '- ?  key\n'
                   '  :  value\n', conf, problem1=(2, 5), problem2=(3, 5))

    def test_after_max(self):
        conf = 'colons: {max-spaces-before: -1, max-spaces-after: 3}'
        self.check('---\n'
                   'object:\n'
                   '  k1:  [a, b]\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k1:    [a, b]\n', conf, problem=(3, 9))
        self.check('---\n'
                   'object:\n'
                   '  k2:  string\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k2:    string\n', conf, problem=(3, 9))
        self.check('---\n'
                   'object:\n'
                   '  other: {key:  value}\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  other: {key:    value}\n'
                   '...\n', conf, problem=(3, 18))

    def test_after_with_explicit_block_mappings(self):
        conf = 'colons: {max-spaces-before: -1, max-spaces-after: 1}'
        self.check('---\n'
                   'object:\n'
                   '  ? key\n'
                   '  : value\n'
                   '...\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  ? key\n'
                   '  :  value\n'
                   '...\n', conf, problem=(4, 5))

    def test_after_do_not_confound_with_trailing_space(self):
        conf = ('colons: {max-spaces-before: 1, max-spaces-after: 1}\n'
                'trailing-spaces: disable\n')
        self.check('---\n'
                   'trailing:     \n'
                   '  - spaces\n', conf)

    def test_both_before_and_after(self):
        conf = 'colons: {max-spaces-before: 0, max-spaces-after: 1}'
        self.check('---\n'
                   'obj:\n'
                   '  string: text\n'
                   '  k:\n'
                   '    - 8\n'
                   '  k3:\n'
                   '    val\n'
                   '  property: [value]\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  k1 :  v1\n', conf, problem1=(3, 5), problem2=(3, 8))
        self.check('---\n'
                   'obj:\n'
                   '  string:  text\n'
                   '  k :\n'
                   '    - 8\n'
                   '  k3:\n'
                   '    val\n'
                   '  property: {a: 1, b:  2, c : 3}\n', conf,
                   problem1=(3, 11), problem2=(4, 4),
                   problem3=(8, 23), problem4=(8, 28))



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os

from tests.common import RuleTestCase


# This file checks examples from YAML 1.2 specification [1] against yamllint.
#
# [1]: http://www.yaml.org/spec/1.2/spec.html
#
# Example files generated with:
#
#     from bs4 import BeautifulSoup
#     with open('spec.html', encoding='iso-8859-1') as f:
#         soup = BeautifulSoup(f, 'lxml')
#         for ex in soup.find_all('div', class_='example'):
#             title = ex.find('p', class_='title').find('b').get_text()
#             id = '-'.join(title.split('\xa0')[:2])[:-1].lower()
#             span = ex.find('span', class_='database')
#             for br in span.find_all("br"):
#                 br.replace_with("\n")
#             text = text.replace('\u2193', '')    # downwards arrow
#             text = text.replace('\u21d3', '')    # double downwards arrow
#             text = text.replace('\u00b7', ' ')   # visible space
#             text = text.replace('\u21d4', '')    # byte order mark
#             text = text.replace('\u2192', '\t')  # right arrow
#             text = text.replace('\u00b0', '')    # empty scalar
#             with open('tests/yaml-1.2-spec-examples/%s' % id, 'w',
#                       encoding='utf-8') as g:
#                 g.write(text)

class SpecificationTestCase(RuleTestCase):
    rule_id = None


conf_general = ('document-start: disable\n'
                'comments: {min-spaces-from-content: 1}\n'
                'braces: {min-spaces-inside: 1, max-spaces-inside: 1}\n'
                'brackets: {min-spaces-inside: 1, max-spaces-inside: 1}\n')
conf_overrides = {
    'example-2.2': 'colons: {max-spaces-after: 2}\n',
    'example-2.4': 'colons: {max-spaces-after: 3}\n',
    'example-2.5': ('empty-lines: {max-end: 2}\n'
                    'brackets: {min-spaces-inside: 0, max-spaces-inside: 2}\n'
                    'commas: {max-spaces-before: -1}\n'),
    'example-2.6': ('braces: {min-spaces-inside: 0, max-spaces-inside: 0}\n'
                    'indentation: disable\n'),
    'example-2.12': ('empty-lines: {max-end: 1}\n'
                     'colons: {max-spaces-before: -1}\n'),
    'example-2.16': 'empty-lines: {max-end: 1}\n',
    'example-2.18': 'empty-lines: {max-end: 1}\n',
    'example-2.19': 'empty-lines: {max-end: 1}\n',
    'example-2.28': 'empty-lines: {max-end: 3}\n',
    'example-5.3': ('indentation: {indent-sequences: false}\n'
                    'colons: {max-spaces-before: 1}\n'),
    'example-6.4': 'trailing-spaces: disable\n',
    'example-6.5': 'trailing-spaces: disable\n',
    'example-6.6': 'trailing-spaces: disable\n',
    'example-6.7': 'trailing-spaces: disable\n',
    'example-6.8': 'trailing-spaces: disable\n',
    'example-6.10': ('empty-lines: {max-end: 2}\n'
                     'trailing-spaces: disable\n'
                     'comments-indentation: disable\n'),
    'example-6.11': ('empty-lines: {max-end: 1}\n'
                     'comments-indentation: disable\n'),
    'example-6.13': 'comments-indentation: disable\n',
    'example-6.14': 'comments-indentation: disable\n',
    'example-6.23': 'colons: {max-spaces-before: 1}\n',
    'example-7.4': ('colons: {max-spaces-before: 1}\n'
                    'indentation: disable\n'),
    'example-7.5': 'trailing-spaces: disable\n',
    'example-7.6': 'trailing-spaces: disable\n',
    'example-7.7': 'indentation: disable\n',
    'example-7.8': ('colons: {max-spaces-before: 1}\n'
                    'indentation: disable\n'),
    'example-7.9': 'trailing-spaces: disable\n',
    'example-7.11': ('colons: {max-spaces-before: 1}\n'
                     'indentation: disable\n'),
    'example-7.13': ('brackets: {min-spaces-inside: 0, max-spaces-inside: 1}\n'
                     'commas: {max-spaces-before: 1, min-spaces-after: 0}\n'),
    'example-7.14': 'indentation: disable\n',
    'example-7.15': ('braces: {min-spaces-inside: 0, max-spaces-inside: 1}\n'
                     'commas: {max-spaces-before: 1, min-spaces-after: 0}\n'
                     'colons: {max-spaces-before: 1}\n'),
    'example-7.16': 'indentation: disable\n',
    'example-7.17': 'indentation: disable\n',
    'example-7.18': 'indentation: disable\n',
    'example-7.19': 'indentation: disable\n',
    'example-7.20': ('colons: {max-spaces-before: 1}\n'
                     'indentation: disable\n'),
    'example-8.1': 'empty-lines: {max-end: 1}\n',
    'example-8.2': 'trailing-spaces: disable\n',
    'example-8.5': ('comments-indentation: disable\n'
                    'trailing-spaces: disable\n'),
    'example-8.6': 'empty-lines: {max-end: 1}\n',
    'example-8.7': 'empty-lines: {max-end: 1}\n',
    'example-8.8': 'trailing-spaces: disable\n',
    'example-8.9': 'empty-lines: {max-end: 1}\n',
    'example-8.14': 'colons: {max-spaces-before: 1}\n',
    'example-8.16': 'indentation: {spaces: 1}\n',
    'example-8.17': 'indentation: disable\n',
    'example-8.20': ('indentation: {indent-sequences: false}\n'
                     'colons: {max-spaces-before: 1}\n'),
    'example-8.22': 'indentation: disable\n',
    'example-10.1': 'colons: {max-spaces-before: 2}\n',
    'example-10.2': 'indentation: {indent-sequences: false}\n',
    'example-10.8': 'truthy: disable\n',
    'example-10.9': 'truthy: disable\n',
}

files = os.listdir(os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                'yaml-1.2-spec-examples'))
assert len(files) == 132


def _gen_test(buffer, conf):
    def test(self):
        self.check(buffer, conf)
    return test


# The following tests are blacklisted (i.e. will not be checked against
# yamllint), because pyyaml is currently not able to parse the contents
# (using yaml.parse()).
pyyaml_blacklist = (
    'example-2.11',
    'example-2.23',
    'example-2.24',
    'example-2.27',
    'example-5.10',
    'example-5.12',
    'example-5.13',
    'example-5.14',
    'example-5.6',
    'example-6.1',
    'example-6.12',
    'example-6.15',
    'example-6.17',
    'example-6.18',
    'example-6.19',
    'example-6.2',
    'example-6.20',
    'example-6.21',
    'example-6.22',
    'example-6.24',
    'example-6.25',
    'example-6.26',
    'example-6.27',
    'example-6.3',
    'example-7.1',
    'example-7.10',
    'example-7.12',
    'example-7.17',
    'example-7.2',
    'example-7.21',
    'example-7.22',
    'example-7.3',
    'example-8.18',
    'example-8.19',
    'example-8.21',
    'example-8.3',
    'example-9.3',
    'example-9.4',
    'example-9.5',
)

for file in files:
    if file in pyyaml_blacklist:
        continue

    with open('tests/yaml-1.2-spec-examples/' + file, encoding='utf-8') as f:
        conf = conf_general + conf_overrides.get(file, '')
        setattr(SpecificationTestCase, 'test_' + file,
                _gen_test(f.read(), conf))



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class LineLengthTestCase(RuleTestCase):
    rule_id = 'line-length'

    def test_disabled(self):
        conf = ('line-length: disable\n'
                'empty-lines: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('---\n', conf)
        self.check(81 * 'a', conf)
        self.check('---\n' + 81 * 'a' + '\n', conf)
        self.check(1000 * 'b', conf)
        self.check('---\n' + 1000 * 'b' + '\n', conf)
        self.check('content: |\n'
                   '  {% this line is' + 99 * ' really' + ' long %}\n',
                   conf)

    def test_default(self):
        conf = ('line-length: {max: 80}\n'
                'empty-lines: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('---\n', conf)
        self.check(80 * 'a', conf)
        self.check('---\n' + 80 * 'a' + '\n', conf)
        self.check(16 * 'aaaa ' + 'z', conf, problem=(1, 81))
        self.check('---\n' + 16 * 'aaaa ' + 'z' + '\n', conf, problem=(2, 81))
        self.check(1000 * 'word ' + 'end', conf, problem=(1, 81))
        self.check('---\n' + 1000 * 'word ' + 'end\n', conf, problem=(2, 81))

    def test_max_length_10(self):
        conf = ('line-length: {max: 10}\n'
                'new-line-at-end-of-file: disable\n')
        self.check('---\nABCD EFGHI', conf)
        self.check('---\nABCD EFGHIJ', conf, problem=(2, 11))
        self.check('---\nABCD EFGHIJ\n', conf, problem=(2, 11))

    def test_spaces(self):
        conf = ('line-length: {max: 80}\n'
                'new-line-at-end-of-file: disable\n'
                'trailing-spaces: disable\n')
        self.check('---\n' + 81 * ' ', conf, problem=(2, 81))
        self.check('---\n' + 81 * ' ' + '\n', conf, problem=(2, 81))

    def test_non_breakable_word(self):
        conf = 'line-length: {max: 20, allow-non-breakable-words: true}'
        self.check('---\n' + 30 * 'A' + '\n', conf)
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        http://localhost/very/long/url\n'
                   '...\n', conf)
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        # http://localhost/very/long/url\n'
                   '        comment\n'
                   '...\n', conf)
        self.check('---\n'
                   'this:\n'
                   'is:\n'
                   'another:\n'
                   '  - https://localhost/very/very/long/url\n'
                   '...\n', conf)
        self.check('---\n'
                   'long_line: http://localhost/very/very/long/url\n', conf,
                   problem=(2, 21))

        conf = 'line-length: {max: 20, allow-non-breakable-words: false}'
        self.check('---\n' + 30 * 'A' + '\n', conf, problem=(2, 21))
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        http://localhost/very/long/url\n'
                   '...\n', conf, problem=(5, 21))
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        # http://localhost/very/long/url\n'
                   '        comment\n'
                   '...\n', conf, problem=(5, 21))
        self.check('---\n'
                   'this:\n'
                   'is:\n'
                   'another:\n'
                   '  - https://localhost/very/very/long/url\n'
                   '...\n', conf, problem=(5, 21))
        self.check('---\n'
                   'long_line: http://localhost/very/very/long/url\n'
                   '...\n', conf, problem=(2, 21))

        conf = 'line-length: {max: 20, allow-non-breakable-words: true}'
        self.check('---\n'
                   '# http://www.verylongurlurlurlurlurlurlurlurl.com\n'
                   'key:\n'
                   '  subkey: value\n', conf)
        self.check('---\n'
                   '## http://www.verylongurlurlurlurlurlurlurlurl.com\n'
                   'key:\n'
                   '  subkey: value\n', conf)
        self.check('---\n'
                   '# # http://www.verylongurlurlurlurlurlurlurlurl.com\n'
                   'key:\n'
                   '  subkey: value\n', conf,
                   problem=(2, 21))
        self.check('---\n'
                   '#A http://www.verylongurlurlurlurlurlurlurlurl.com\n'
                   'key:\n'
                   '  subkey: value\n', conf,
                   problem1=(2, 2, 'comments'),
                   problem2=(2, 21, 'line-length'))

        conf = ('line-length: {max: 20, allow-non-breakable-words: true}\n'
                'trailing-spaces: disable')
        self.check('---\n'
                   'loooooooooong+word+and+some+space+at+the+end       \n',
                   conf, problem=(2, 21))

    def test_non_breakable_inline_mappings(self):
        conf = 'line-length: {max: 20, ' \
               'allow-non-breakable-inline-mappings: true}'
        self.check('---\n'
                   'long_line: http://localhost/very/very/long/url\n'
                   'long line: http://localhost/very/very/long/url\n', conf)
        self.check('---\n'
                   '- long line: http://localhost/very/very/long/url\n', conf)

        self.check('---\n'
                   'long_line: http://localhost/short/url + word\n'
                   'long line: http://localhost/short/url + word\n',
                   conf, problem1=(2, 21), problem2=(3, 21))

        conf = ('line-length: {max: 20,'
                '              allow-non-breakable-inline-mappings: true}\n'
                'trailing-spaces: disable')
        self.check('---\n'
                   'long_line: and+some+space+at+the+end       \n',
                   conf, problem=(2, 21))
        self.check('---\n'
                   'long line: and+some+space+at+the+end       \n',
                   conf, problem=(2, 21))
        self.check('---\n'
                   '- long line: and+some+space+at+the+end       \n',
                   conf, problem=(2, 21))

        # See https://github.com/adrienverge/yamllint/issues/21
        conf = 'line-length: {allow-non-breakable-inline-mappings: true}'
        self.check('---\n'
                   'content: |\n'
                   '  {% this line is' + 99 * ' really' + ' long %}\n',
                   conf, problem=(3, 81))

    def test_unicode(self):
        conf = 'line-length: {max: 53}'
        self.check('---\n'
                   '# This is a test to check if “line-length” works nice\n'
                   'with: “unicode characters” that span across bytes! ↺\n',
                   conf)
        conf = 'line-length: {max: 51}'
        self.check('---\n'
                   '# This is a test to check if “line-length” works nice\n'
                   'with: “unicode characters” that span across bytes! ↺\n',
                   conf, problem1=(2, 52), problem2=(3, 52))

    def test_with_dos_newlines(self):
        conf = ('line-length: {max: 10}\n'
                'new-lines: {type: dos}\n'
                'new-line-at-end-of-file: disable\n')
        self.check('---\r\nABCD EFGHI', conf)
        self.check('---\r\nABCD EFGHI\r\n', conf)
        self.check('---\r\nABCD EFGHIJ', conf, problem=(2, 11))
        self.check('---\r\nABCD EFGHIJ\r\n', conf, problem=(2, 11))



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class CommaTestCase(RuleTestCase):
    rule_id = 'commas'

    def test_disabled(self):
        conf = 'commas: disable'
        self.check('---\n'
                   'dict: {a: b ,   c: "1 2 3",    d: e , f: [g,      h]}\n'
                   'array: [\n'
                   '  elem  ,\n'
                   '  key: val ,\n'
                   ']\n'
                   'map: {\n'
                   '  key1: val1 ,\n'
                   '  key2: val2,\n'
                   '}\n'
                   '...\n', conf)
        self.check('---\n'
                   '- [one, two , three,four]\n'
                   '- {five,six , seven, eight}\n'
                   '- [\n'
                   '  nine,  ten\n'
                   '  , eleven\n'
                   '  ,twelve\n'
                   ']\n'
                   '- {\n'
                   '  thirteen: 13,  fourteen\n'
                   '  , fifteen: 15\n'
                   '  ,sixteen: 16\n'
                   '}\n', conf)

    def test_before_max(self):
        conf = ('commas:\n'
                '  max-spaces-before: 0\n'
                '  min-spaces-after: 0\n'
                '  max-spaces-after: -1\n')
        self.check('---\n'
                   'array: [1, 2,  3, 4]\n'
                   '...\n', conf)
        self.check('---\n'
                   'array: [1, 2 ,  3, 4]\n'
                   '...\n', conf, problem=(2, 13))
        self.check('---\n'
                   'array: [1 , 2,  3      , 4]\n'
                   '...\n', conf, problem1=(2, 10), problem2=(2, 23))
        self.check('---\n'
                   'dict: {a: b, c: "1 2 3", d: e,  f: [g, h]}\n'
                   '...\n', conf)
        self.check('---\n'
                   'dict: {a: b, c: "1 2 3" , d: e,  f: [g, h]}\n'
                   '...\n', conf, problem=(2, 24))
        self.check('---\n'
                   'dict: {a: b , c: "1 2 3", d: e,  f: [g    , h]}\n'
                   '...\n', conf, problem1=(2, 12), problem2=(2, 42))
        self.check('---\n'
                   'array: [\n'
                   '  elem,\n'
                   '  key: val,\n'
                   ']\n', conf)
        self.check('---\n'
                   'array: [\n'
                   '  elem ,\n'
                   '  key: val,\n'
                   ']\n', conf, problem=(3, 7))
        self.check('---\n'
                   'map: {\n'
                   '  key1: val1,\n'
                   '  key2: val2,\n'
                   '}\n', conf)
        self.check('---\n'
                   'map: {\n'
                   '  key1: val1,\n'
                   '  key2: val2 ,\n'
                   '}\n', conf, problem=(4, 13))

    def test_before_max_with_comma_on_new_line(self):
        conf = ('commas:\n'
                '  max-spaces-before: 0\n'
                '  min-spaces-after: 0\n'
                '  max-spaces-after: -1\n')
        self.check('---\n'
                   'flow-seq: [1, 2, 3\n'
                   '           , 4, 5, 6]\n'
                   '...\n', conf, problem=(3, 11))
        self.check('---\n'
                   'flow-map: {a: 1, b: 2\n'
                   '           , c: 3}\n'
                   '...\n', conf, problem=(3, 11))

        conf = ('commas:\n'
                '  max-spaces-before: 0\n'
                '  min-spaces-after: 0\n'
                '  max-spaces-after: -1\n'
                'indentation: disable\n')
        self.check('---\n'
                   'flow-seq: [1, 2, 3\n'
                   '         , 4, 5, 6]\n'
                   '...\n', conf, problem=(3, 9))
        self.check('---\n'
                   'flow-map: {a: 1, b: 2\n'
                   '         , c: 3}\n'
                   '...\n', conf, problem=(3, 9))
        self.check('---\n'
                   '[\n'
                   '1,\n'
                   '2\n'
                   ', 3\n'
                   ']\n', conf, problem=(5, 1))
        self.check('---\n'
                   '{\n'
                   'a: 1,\n'
                   'b: 2\n'
                   ', c: 3\n'
                   '}\n', conf, problem=(5, 1))

    def test_before_max_3(self):
        conf = ('commas:\n'
                '  max-spaces-before: 3\n'
                '  min-spaces-after: 0\n'
                '  max-spaces-after: -1\n')
        self.check('---\n'
                   'array: [1 , 2, 3   , 4]\n'
                   '...\n', conf)
        self.check('---\n'
                   'array: [1 , 2, 3    , 4]\n'
                   '...\n', conf, problem=(2, 20))
        self.check('---\n'
                   'array: [\n'
                   '  elem1   ,\n'
                   '  elem2    ,\n'
                   '  key: val,\n'
                   ']\n', conf, problem=(4, 11))

    def test_after_min(self):
        conf = ('commas:\n'
                '  max-spaces-before: -1\n'
                '  min-spaces-after: 1\n'
                '  max-spaces-after: -1\n')
        self.check('---\n'
                   '- [one, two , three,four]\n'
                   '- {five,six , seven, eight}\n'
                   '- [\n'
                   '  nine,  ten\n'
                   '  , eleven\n'
                   '  ,twelve\n'
                   ']\n'
                   '- {\n'
                   '  thirteen: 13,  fourteen\n'
                   '  , fifteen: 15\n'
                   '  ,sixteen: 16\n'
                   '}\n', conf,
                   problem1=(2, 21), problem2=(3, 9),
                   problem3=(7, 4), problem4=(12, 4))

    def test_after_max(self):
        conf = ('commas:\n'
                '  max-spaces-before: -1\n'
                '  min-spaces-after: 0\n'
                '  max-spaces-after: 1\n')
        self.check('---\n'
                   'array: [1, 2, 3, 4]\n'
                   '...\n', conf)
        self.check('---\n'
                   'array: [1, 2,  3, 4]\n'
                   '...\n', conf, problem=(2, 15))
        self.check('---\n'
                   'array: [1,  2, 3,     4]\n'
                   '...\n', conf, problem1=(2, 12), problem2=(2, 22))
        self.check('---\n'
                   'dict: {a: b , c: "1 2 3", d: e, f: [g, h]}\n'
                   '...\n', conf)
        self.check('---\n'
                   'dict: {a: b , c: "1 2 3",  d: e, f: [g, h]}\n'
                   '...\n', conf, problem=(2, 27))
        self.check('---\n'
                   'dict: {a: b ,  c: "1 2 3", d: e, f: [g,     h]}\n'
                   '...\n', conf, problem1=(2, 15), problem2=(2, 44))
        self.check('---\n'
                   'array: [\n'
                   '  elem,\n'
                   '  key: val,\n'
                   ']\n', conf)
        self.check('---\n'
                   'array: [\n'
                   '  elem,  key: val,\n'
                   ']\n', conf, problem=(3, 9))
        self.check('---\n'
                   'map: {\n'
                   '  key1: val1,   key2: [val2,  val3]\n'
                   '}\n', conf, problem1=(3, 16), problem2=(3, 30))

    def test_after_max_3(self):
        conf = ('commas:\n'
                '  max-spaces-before: -1\n'
                '  min-spaces-after: 1\n'
                '  max-spaces-after: 3\n')
        self.check('---\n'
                   'array: [1,  2, 3,   4]\n'
                   '...\n', conf)
        self.check('---\n'
                   'array: [1,  2, 3,    4]\n'
                   '...\n', conf, problem=(2, 21))
        self.check('---\n'
                   'dict: {a: b ,   c: "1 2 3",    d: e, f: [g,      h]}\n'
                   '...\n', conf, problem1=(2, 31), problem2=(2, 49))

    def test_both_before_and_after(self):
        conf = ('commas:\n'
                '  max-spaces-before: 0\n'
                '  min-spaces-after: 1\n'
                '  max-spaces-after: 1\n')
        self.check('---\n'
                   'dict: {a: b ,   c: "1 2 3",    d: e , f: [g,      h]}\n'
                   'array: [\n'
                   '  elem  ,\n'
                   '  key: val ,\n'
                   ']\n'
                   'map: {\n'
                   '  key1: val1 ,\n'
                   '  key2: val2,\n'
                   '}\n'
                   '...\n', conf,
                   problem1=(2, 12), problem2=(2, 16), problem3=(2, 31),
                   problem4=(2, 36), problem5=(2, 50), problem6=(4, 8),
                   problem7=(5, 11), problem8=(8, 13))
        conf = ('commas:\n'
                '  max-spaces-before: 0\n'
                '  min-spaces-after: 1\n'
                '  max-spaces-after: 1\n'
                'indentation: disable\n')
        self.check('---\n'
                   '- [one, two , three,four]\n'
                   '- {five,six , seven, eight}\n'
                   '- [\n'
                   '  nine,  ten\n'
                   '  , eleven\n'
                   '  ,twelve\n'
                   ']\n'
                   '- {\n'
                   '  thirteen: 13,  fourteen\n'
                   '  , fifteen: 15\n'
                   '  ,sixteen: 16\n'
                   '}\n', conf,
                   problem1=(2, 12), problem2=(2, 21), problem3=(3, 9),
                   problem4=(3, 12), problem5=(5, 9), problem6=(6, 2),
                   problem7=(7, 2), problem8=(7, 4), problem9=(10, 17),
                   problem10=(11, 2), problem11=(12, 2), problem12=(12, 4))



# Copyright (C) 2016 Peter Ericson
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to forbid non-explicitly typed truthy values other than allowed
ones (by default: ``true`` and ``false``), for example ``YES`` or ``off``.

This can be useful to prevent surprises from YAML parsers transforming
``[yes, FALSE, Off]`` into ``[true, false, false]`` or
``{y: 1, yes: 2, on: 3, true: 4, True: 5}`` into ``{y: 1, true: 5}``.

.. rubric:: Options

* ``allowed-values`` defines the list of truthy values which will be ignored
  during linting. The default is ``['true', 'false']``, but can be changed to
  any list containing: ``'TRUE'``, ``'True'``,  ``'true'``, ``'FALSE'``,
  ``'False'``, ``'false'``, ``'YES'``, ``'Yes'``, ``'yes'``, ``'NO'``,
  ``'No'``, ``'no'``, ``'ON'``, ``'On'``, ``'on'``, ``'OFF'``, ``'Off'``,
  ``'off'``.
* ``check-keys`` disables verification for keys in mappings. By default,
  ``truthy`` rule applies to both keys and values. Set this option to ``false``
  to prevent this.

.. rubric:: Default values (when enabled)

.. code-block:: yaml

 rules:
   truthy:
     allowed-values: ['true', 'false']
     check-keys: true

.. rubric:: Examples

#. With ``truthy: {}``

   the following code snippet would **PASS**:
   ::

    boolean: true

    object: {"True": 1, 1: "True"}

    "yes":  1
    "on":   2
    "True": 3

     explicit:
       string1: !!str True
       string2: !!str yes
       string3: !!str off
       encoded: !!binary |
                  True
                  OFF
                  pad==  # this decodes as 'N\xbb\x9e8Qii'
       boolean1: !!bool true
       boolean2: !!bool "false"
       boolean3: !!bool FALSE
       boolean4: !!bool True
       boolean5: !!bool off
       boolean6: !!bool NO

   the following code snippet would **FAIL**:
   ::

    object: {True: 1, 1: True}

   the following code snippet would **FAIL**:
   ::

    yes:  1
    on:   2
    True: 3

#. With ``truthy: {allowed-values: ["yes", "no"]}``

   the following code snippet would **PASS**:
   ::

    - yes
    - no
    - "true"
    - 'false'
    - foo
    - bar

   the following code snippet would **FAIL**:
   ::

    - true
    - false
    - on
    - off

#. With ``truthy: {check-keys: false}``

   the following code snippet would **PASS**:
   ::

    yes:  1
    on:   2
    true: 3

   the following code snippet would **FAIL**:
   ::

    yes:  Yes
    on:   On
    true: True
"""

import yaml

from yamllint.linter import LintProblem


TRUTHY = ['YES', 'Yes', 'yes',
          'NO', 'No', 'no',
          'TRUE', 'True', 'true',
          'FALSE', 'False', 'false',
          'ON', 'On', 'on',
          'OFF', 'Off', 'off']


ID = 'truthy'
TYPE = 'token'
CONF = {'allowed-values': list(TRUTHY), 'check-keys': bool}
DEFAULT = {'allowed-values': ['true', 'false'], 'check-keys': True}


def check(conf, token, prev, next, nextnext, context):
    if prev and isinstance(prev, yaml.tokens.TagToken):
        return

    if (not conf['check-keys'] and isinstance(prev, yaml.tokens.KeyToken) and
            isinstance(token, yaml.tokens.ScalarToken)):
        return

    if isinstance(token, yaml.tokens.ScalarToken):
        if (token.value in (set(TRUTHY) - set(conf['allowed-values'])) and
                token.style is None):
            yield LintProblem(token.start_mark.line + 1,
                              token.start_mark.column + 1,
                              "truthy value should be one of [" +
                              ", ".join(sorted(conf['allowed-values'])) + "]")



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import argparse
import locale
import os
import platform
import sys

from yamllint import APP_DESCRIPTION, APP_NAME, APP_VERSION
from yamllint import linter
from yamllint.config import YamlLintConfig, YamlLintConfigError
from yamllint.linter import PROBLEM_LEVELS


def find_files_recursively(items, conf):
    for item in items:
        if os.path.isdir(item):
            for root, dirnames, filenames in os.walk(item):
                for f in filenames:
                    filepath = os.path.join(root, f)
                    if conf.is_yaml_file(filepath):
                        yield filepath
        else:
            yield item


def supports_color():
    supported_platform = not (platform.system() == 'Windows' and not
                              ('ANSICON' in os.environ or
                               ('TERM' in os.environ and
                                os.environ['TERM'] == 'ANSI')))
    return (supported_platform and
            hasattr(sys.stdout, 'isatty') and sys.stdout.isatty())


class Format:
    @staticmethod
    def parsable(problem, filename):
        return ('%(file)s:%(line)s:%(column)s: [%(level)s] %(message)s' %
                {'file': filename,
                 'line': problem.line,
                 'column': problem.column,
                 'level': problem.level,
                 'message': problem.message})

    @staticmethod
    def standard(problem, filename):
        line = '  %d:%d' % (problem.line, problem.column)
        line += max(12 - len(line), 0) * ' '
        line += problem.level
        line += max(21 - len(line), 0) * ' '
        line += problem.desc
        if problem.rule:
            line += '  (%s)' % problem.rule
        return line

    @staticmethod
    def standard_color(problem, filename):
        line = '  \033[2m%d:%d\033[0m' % (problem.line, problem.column)
        line += max(20 - len(line), 0) * ' '
        if problem.level == 'warning':
            line += '\033[33m%s\033[0m' % problem.level
        else:
            line += '\033[31m%s\033[0m' % problem.level
        line += max(38 - len(line), 0) * ' '
        line += problem.desc
        if problem.rule:
            line += '  \033[2m(%s)\033[0m' % problem.rule
        return line

    @staticmethod
    def github(problem, filename):
        line = '::'
        line += problem.level
        line += ' file=' + filename + ','
        line += 'line=' + format(problem.line) + ','
        line += 'col=' + format(problem.column)
        line += '::'
        line += format(problem.line)
        line += ':'
        line += format(problem.column)
        line += ' '
        if problem.rule:
            line += '[' + problem.rule + '] '
        line += problem.desc
        return line


def show_problems(problems, file, args_format, no_warn):
    max_level = 0
    first = True

    if args_format == 'auto':
        if ('GITHUB_ACTIONS' in os.environ and
                'GITHUB_WORKFLOW' in os.environ):
            args_format = 'github'
        elif supports_color():
            args_format = 'colored'

    for problem in problems:
        max_level = max(max_level, PROBLEM_LEVELS[problem.level])
        if no_warn and (problem.level != 'error'):
            continue
        if args_format == 'parsable':
            print(Format.parsable(problem, file))
        elif args_format == 'github':
            if first:
                print('::group::%s' % file)
                first = False
            print(Format.github(problem, file))
        elif args_format == 'colored':
            if first:
                print('\033[4m%s\033[0m' % file)
                first = False
            print(Format.standard_color(problem, file))
        else:
            if first:
                print(file)
                first = False
            print(Format.standard(problem, file))

    if not first and args_format == 'github':
        print('::endgroup::')

    if not first and args_format != 'parsable':
        print('')

    return max_level


def run(argv=None):
    parser = argparse.ArgumentParser(prog=APP_NAME,
                                     description=APP_DESCRIPTION)
    files_group = parser.add_mutually_exclusive_group(required=True)
    files_group.add_argument('files', metavar='FILE_OR_DIR', nargs='*',
                             default=(),
                             help='files to check')
    files_group.add_argument('-', action='store_true', dest='stdin',
                             help='read from standard input')
    config_group = parser.add_mutually_exclusive_group()
    config_group.add_argument('-c', '--config-file', dest='config_file',
                              action='store',
                              help='path to a custom configuration')
    config_group.add_argument('-d', '--config-data', dest='config_data',
                              action='store',
                              help='custom configuration (as YAML source)')
    parser.add_argument('--list-files', action='store_true', dest='list_files',
                        help='list files to lint and exit')
    parser.add_argument('-f', '--format',
                        choices=('parsable', 'standard', 'colored', 'github',
                                 'auto'),
                        default='auto', help='format for parsing output')
    parser.add_argument('-s', '--strict',
                        action='store_true',
                        help='return non-zero exit code on warnings '
                             'as well as errors')
    parser.add_argument('--no-warnings',
                        action='store_true',
                        help='output only error level problems')
    parser.add_argument('-v', '--version', action='version',
                        version='{} {}'.format(APP_NAME, APP_VERSION))

    args = parser.parse_args(argv)

    if 'YAMLLINT_CONFIG_FILE' in os.environ:
        user_global_config = os.path.expanduser(
            os.environ['YAMLLINT_CONFIG_FILE'])
    # User-global config is supposed to be in ~/.config/yamllint/config
    elif 'XDG_CONFIG_HOME' in os.environ:
        user_global_config = os.path.join(
            os.environ['XDG_CONFIG_HOME'], 'yamllint', 'config')
    else:
        user_global_config = os.path.expanduser('~/.config/yamllint/config')

    try:
        if args.config_data is not None:
            if args.config_data != '' and ':' not in args.config_data:
                args.config_data = 'extends: ' + args.config_data
            conf = YamlLintConfig(content=args.config_data)
        elif args.config_file is not None:
            conf = YamlLintConfig(file=args.config_file)
        elif os.path.isfile('.yamllint'):
            conf = YamlLintConfig(file='.yamllint')
        elif os.path.isfile('.yamllint.yaml'):
            conf = YamlLintConfig(file='.yamllint.yaml')
        elif os.path.isfile('.yamllint.yml'):
            conf = YamlLintConfig(file='.yamllint.yml')
        elif os.path.isfile(user_global_config):
            conf = YamlLintConfig(file=user_global_config)
        else:
            conf = YamlLintConfig('extends: default')
    except YamlLintConfigError as e:
        print(e, file=sys.stderr)
        sys.exit(-1)

    if conf.locale is not None:
        locale.setlocale(locale.LC_ALL, conf.locale)

    if args.list_files:
        for file in find_files_recursively(args.files, conf):
            if not conf.is_file_ignored(file):
                print(file)
        sys.exit(0)

    max_level = 0

    for file in find_files_recursively(args.files, conf):
        filepath = file[2:] if file.startswith('./') else file
        try:
            with open(file, newline='') as f:
                problems = linter.run(f, conf, filepath)
        except OSError as e:
            print(e, file=sys.stderr)
            sys.exit(-1)
        prob_level = show_problems(problems, file, args_format=args.format,
                                   no_warn=args.no_warnings)
        max_level = max(max_level, prob_level)

    # read yaml from stdin
    if args.stdin:
        try:
            problems = linter.run(sys.stdin, conf, '')
        except OSError as e:
            print(e, file=sys.stderr)
            sys.exit(-1)
        prob_level = show_problems(problems, 'stdin', args_format=args.format,
                                   no_warn=args.no_warnings)
        max_level = max(max_level, prob_level)

    if max_level == PROBLEM_LEVELS['error']:
        return_code = 1
    elif max_level == PROBLEM_LEVELS['warning']:
        return_code = 2 if args.strict else 0
    else:
        return_code = 0

    sys.exit(return_code)



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the position and formatting of comments.

.. rubric:: Options

* Use ``require-starting-space`` to require a space character right after the
  ``#``. Set to ``true`` to enable, ``false`` to disable.
* Use ``ignore-shebangs`` to ignore a
  `shebang <https://en.wikipedia.org/wiki/Shebang_(Unix)>`_ at the beginning of
  the file when ``require-starting-space`` is set.
* ``min-spaces-from-content`` is used to visually separate inline comments from
  content. It defines the minimal required number of spaces between a comment
  and its preceding content.

.. rubric:: Default values (when enabled)

.. code-block:: yaml

 rules:
   comments:
     require-starting-space: true
     ignore-shebangs: true
     min-spaces-from-content: 2

.. rubric:: Examples

#. With ``comments: {require-starting-space: true}``

   the following code snippet would **PASS**:
   ::

    # This sentence
    # is a block comment

   the following code snippet would **PASS**:
   ::

    ##############################
    ## This is some documentation

   the following code snippet would **FAIL**:
   ::

    #This sentence
    #is a block comment

#. With ``comments: {min-spaces-from-content: 2}``

   the following code snippet would **PASS**:
   ::

    x = 2 ^ 127 - 1  # Mersenne prime number

   the following code snippet would **FAIL**:
   ::

    x = 2 ^ 127 - 1 # Mersenne prime number
"""


from yamllint.linter import LintProblem


ID = 'comments'
TYPE = 'comment'
CONF = {'require-starting-space': bool,
        'ignore-shebangs': bool,
        'min-spaces-from-content': int}
DEFAULT = {'require-starting-space': True,
           'ignore-shebangs': True,
           'min-spaces-from-content': 2}


def check(conf, comment):
    if (conf['min-spaces-from-content'] != -1 and comment.is_inline() and
            comment.pointer - comment.token_before.end_mark.pointer <
            conf['min-spaces-from-content']):
        yield LintProblem(comment.line_no, comment.column_no,
                          'too few spaces before comment')

    if conf['require-starting-space']:
        text_start = comment.pointer + 1
        while (comment.buffer[text_start] == '#' and
               text_start < len(comment.buffer)):
            text_start += 1
        if text_start < len(comment.buffer):
            if (conf['ignore-shebangs'] and
                    comment.line_no == 1 and
                    comment.column_no == 1 and
                    comment.buffer[text_start] == '!'):
                return
            # We can test for both \r and \r\n just by checking first char
            # \r itself is a valid newline on some older OS.
            elif comment.buffer[text_start] not in {' ', '\n', '\r', '\x00'}:
                column = comment.column_no + text_start - comment.pointer
                yield LintProblem(comment.line_no,
                                  column,
                                  'missing starting space in comment')



# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import fileinput
import os.path

import pathspec
import yaml

import yamllint.rules


class YamlLintConfigError(Exception):
    pass


class YamlLintConfig:
    def __init__(self, content=None, file=None):
        assert (content is None) ^ (file is None)

        self.ignore = None

        self.yaml_files = pathspec.PathSpec.from_lines(
            'gitwildmatch', ['*.yaml', '*.yml', '.yamllint'])

        self.locale = None

        if file is not None:
            with open(file) as f:
                content = f.read()

        self.parse(content)
        self.validate()

    def is_file_ignored(self, filepath):
        return self.ignore and self.ignore.match_file(filepath)

    def is_yaml_file(self, filepath):
        return self.yaml_files.match_file(os.path.basename(filepath))

    def enabled_rules(self, filepath):
        return [yamllint.rules.get(id) for id, val in self.rules.items()
                if val is not False and (
                    filepath is None or 'ignore' not in val or
                    not val['ignore'].match_file(filepath))]

    def extend(self, base_config):
        assert isinstance(base_config, YamlLintConfig)

        for rule in self.rules:
            if (isinstance(self.rules[rule], dict) and
                    rule in base_config.rules and
                    base_config.rules[rule] is not False):
                base_config.rules[rule].update(self.rules[rule])
            else:
                base_config.rules[rule] = self.rules[rule]

        self.rules = base_config.rules

        if base_config.ignore is not None:
            self.ignore = base_config.ignore

    def parse(self, raw_content):
        try:
            conf = yaml.safe_load(raw_content)
        except Exception as e:
            raise YamlLintConfigError('invalid config: %s' % e)

        if not isinstance(conf, dict):
            raise YamlLintConfigError('invalid config: not a dict')

        self.rules = conf.get('rules', {})
        for rule in self.rules:
            if self.rules[rule] == 'enable':
                self.rules[rule] = {}
            elif self.rules[rule] == 'disable':
                self.rules[rule] = False

        # Does this conf override another conf that we need to load?
        if 'extends' in conf:
            path = get_extended_config_file(conf['extends'])
            base = YamlLintConfig(file=path)
            try:
                self.extend(base)
            except Exception as e:
                raise YamlLintConfigError('invalid config: %s' % e)

        if 'ignore' in conf and 'ignore-from-file' in conf:
            raise YamlLintConfigError(
                'invalid config: ignore and ignore-from-file keys cannot be '
                'used together')
        elif 'ignore-from-file' in conf:
            if isinstance(conf['ignore-from-file'], str):
                conf['ignore-from-file'] = [conf['ignore-from-file']]
            if not (isinstance(conf['ignore-from-file'], list) and all(
                    isinstance(ln, str) for ln in conf['ignore-from-file'])):
                raise YamlLintConfigError(
                    'invalid config: ignore-from-file should contain '
                    'filename(s), either as a list or string')
            with fileinput.input(conf['ignore-from-file']) as f:
                self.ignore = pathspec.PathSpec.from_lines('gitwildmatch', f)
        elif 'ignore' in conf:
            if isinstance(conf['ignore'], str):
                self.ignore = pathspec.PathSpec.from_lines(
                    'gitwildmatch', conf['ignore'].splitlines())
            elif (isinstance(conf['ignore'], list) and
                    all(isinstance(line, str) for line in conf['ignore'])):
                self.ignore = pathspec.PathSpec.from_lines(
                    'gitwildmatch', conf['ignore'])
            else:
                raise YamlLintConfigError(
                    'invalid config: ignore should contain file patterns')

        if 'yaml-files' in conf:
            if not (isinstance(conf['yaml-files'], list)
                    and all(isinstance(i, str) for i in conf['yaml-files'])):
                raise YamlLintConfigError(
                    'invalid config: yaml-files '
                    'should be a list of file patterns')
            self.yaml_files = pathspec.PathSpec.from_lines('gitwildmatch',
                                                           conf['yaml-files'])

        if 'locale' in conf:
            if not isinstance(conf['locale'], str):
                raise YamlLintConfigError(
                    'invalid config: locale should be a string')
            self.locale = conf['locale']

    def validate(self):
        for id in self.rules:
            try:
                rule = yamllint.rules.get(id)
            except Exception as e:
                raise YamlLintConfigError('invalid config: %s' % e)

            self.rules[id] = validate_rule_conf(rule, self.rules[id])


def validate_rule_conf(rule, conf):
    if conf is False:  # disable
        return False

    if isinstance(conf, dict):
        if ('ignore' in conf and
                not isinstance(conf['ignore'], pathspec.pathspec.PathSpec)):
            if isinstance(conf['ignore'], str):
                conf['ignore'] = pathspec.PathSpec.from_lines(
                    'gitwildmatch', conf['ignore'].splitlines())
            elif (isinstance(conf['ignore'], list) and
                    all(isinstance(line, str) for line in conf['ignore'])):
                conf['ignore'] = pathspec.PathSpec.from_lines(
                    'gitwildmatch', conf['ignore'])
            else:
                raise YamlLintConfigError(
                    'invalid config: ignore should contain file patterns')

        if 'level' not in conf:
            conf['level'] = 'error'
        elif conf['level'] not in ('error', 'warning'):
            raise YamlLintConfigError(
                'invalid config: level should be "error" or "warning"')

        options = getattr(rule, 'CONF', {})
        options_default = getattr(rule, 'DEFAULT', {})
        for optkey in conf:
            if optkey in ('ignore', 'ignore-from-file', 'level'):
                continue
            if optkey not in options:
                raise YamlLintConfigError(
                    'invalid config: unknown option "%s" for rule "%s"' %
                    (optkey, rule.ID))
            # Example: CONF = {option: (bool, 'mixed')}
            #          → {option: true}         → {option: mixed}
            if isinstance(options[optkey], tuple):
                if (conf[optkey] not in options[optkey] and
                        type(conf[optkey]) not in options[optkey]):
                    raise YamlLintConfigError(
                        'invalid config: option "%s" of "%s" should be in %s'
                        % (optkey, rule.ID, options[optkey]))
            # Example: CONF = {option: ['flag1', 'flag2', int]}
            #          → {option: [flag1]}      → {option: [42, flag1, flag2]}
            elif isinstance(options[optkey], list):
                if (type(conf[optkey]) is not list or
                        any(flag not in options[optkey] and
                            type(flag) not in options[optkey]
                            for flag in conf[optkey])):
                    raise YamlLintConfigError(
                        ('invalid config: option "%s" of "%s" should only '
                         'contain values in %s')
                        % (optkey, rule.ID, str(options[optkey])))
            # Example: CONF = {option: int}
            #          → {option: 42}
            else:
                if not isinstance(conf[optkey], options[optkey]):
                    raise YamlLintConfigError(
                        'invalid config: option "%s" of "%s" should be %s'
                        % (optkey, rule.ID, options[optkey].__name__))
        for optkey in options:
            if optkey not in conf:
                conf[optkey] = options_default[optkey]

        if hasattr(rule, 'VALIDATE'):
            res = rule.VALIDATE(conf)
            if res:
                raise YamlLintConfigError('invalid config: %s: %s' %
                                          (rule.ID, res))
    else:
        raise YamlLintConfigError(('invalid config: rule "%s": should be '
                                   'either "enable", "disable" or a dict')
                                  % rule.ID)

    return conf


def get_extended_config_file(name):
    # Is it a standard conf shipped with yamllint...
    if '/' not in name:
        std_conf = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                'conf', name + '.yaml')

        if os.path.isfile(std_conf):
            return std_conf

    # or a custom conf on filesystem?
    return name
